#!/bin/sh
#
#

OBEX_LANG=de

contrib/check_trans.tcl lang/obextool.$OBEX_LANG obextool.tk lib/* etc/*
contrib/check_trans.tcl lang/adr_plug.$OBEX_LANG plugins/adr_plugin.tcl 
contrib/check_trans.tcl lang/vcs_plug.$OBEX_LANG plugins/vcs_plugin.tcl 
contrib/check_trans.tcl lang/sms_siem.$OBEX_LANG plugins/sm*

