#
#  PL language file for Siemens SMS-Plugin
#  Provided by Kamil Nowak
#

array set vcs_plug_Text [list\
  "Category" "Kategoria"\
  "Start" "Start"\
  "Alarm" "Alarm"\
  "Description" "Opis"\
  "Repeat every" "Liczba powtórzeń"\
\
  "Birthday" "Urodziny"\
  "Anniversary" "Rocznica"\
  "Special event" "Szczególna okazja"\
  "Memo" "Pamięć"\
  "Phone call" "Rozmowa telefoniczna"\
  "Meeting" "Spotkanie"\
\
  "day" "dzień"\
  "year" "rok"\
  "month" "miesiąc"\
  "week" "tydzień"\
  "Mo" "Po"\
  "Tu" "Wt"\
  "We" "Śr"\
  "Th" "Cz"\
  "Fr" "Pi"\
  "Sa" "So"\
  "Su" "Ni"\
  "Unexpected Repeat-Rule '%s' found!" "Znaleziono nieoczekiwaną zasadę powtórzeń '%s'!"\
  " at the %s." "o %s."\
  " on%s" " w %s"\
  "%s date" "%s data"\
  "%s time" "%s czas"\
  " on every " " na wszystkie "\
  "VCal Folder: %s" "Folder VCal"\
  "C&reate" "U&twórz"\
  "&Copy" "&Skopiuj" \
  "&Edit" "&Edytuj"\
  "&Delete" "&Usuń"\
  "E&xport..." "E&ksportuj..."\
  "&Close" "&Zamknij"\
  "&Save" "&Zapisz"\
  "&Read all" "Odczytaj wszystkie"\
  "Edit selected data record" "Edytuj zaznaczony rekord"\
  "Create new record using current selection" "Utwórz nowy rekord używając zaznaczonego"\
  "Create new data record" "Utwórz nowy rekord"\
  "Delete selected data record" "Usuń zaznaczony rekord"\
  "Export all records to text file" "Eksportuj wszystkie rekordy do pliku tekstowego"\
  "Save entry to device" "Zapisz wpisy na urządzeniu"\
  "Close list window" "Zamknij okno listy"\
  "Read all entries and open list" "Odczytaj wszystkie wpisy i otwórz listę"\
  "Close detail view window" "Otwórz okno z widokiem szczegółów"\
  "%s window already open!\nPlease" "%s otworzono już okno!\nProszę"\
  " close it before opening a new one." " zamknij je zanim otworzysz nowe."\
  "Unable to download file '%s'!" "Nie można ściągnąć pliku '%s'!"\
  "Stop" "Stop"\
  "Reading all VCal entries..." "Odczytywanie wpisów VCal..."\
  "Reading %s..." "Odczytywanie %s..."\
  "%d entries read." "%d wpisów odczytano."\
  "File does not seem to be a VCal file!" "Wydaje się, że plik nie jest plikiem VCal"\
  "No version info in VCal File!" "Nie znaleziono informacji o wersji w pliku VCal"\
  "This plugin does not support version %s of VCal file!" "Ten plugin nie obsługuje wersji %s pliku VCal"\
  "ASCII Text Files <TAB>" "Pliki tekstowe ASCII <TAB>"\
  "ASCII Files <Semicolon>" "Pliki ASCII <Średnik>"\
  "CSV Files <Comma separated>" "Pliki CSV <Oddzielone przecinkiem>"\
  "All files" "Wszystkie pliki"\
  "Export file" "Eksportuj plik"\
  "%d records written to file '%s'" "%d rekordów zapisanych w pliku '%s'"\
  "VCal detail data" "VCal szczegóły danych"\
  "No valid VCal entry for\n%s: %s" "Nie znaleziono poprawnych wpisów VCal dla\n%s: %s"\
  "Confirm" "Potwierdź"\
  "Do you really want to store this record?" "Czy na pewno chcesz zapisać ten rekord?"\
  "Uploading file '%s' to '%s'..." "Wysyłanie pliku '%s' do '%s'..."\
  "Missing records in stored entry:" "Brakuje rekordów we wpisach:"\
  "Saved entries not identical to input entry:%s" "Zapisane wpisy nie są takie same jak wpisy na wejściu:%s"\
  "File '%s' stored" "Plik '%s' zapisano"\
  "Obextool VCal-Plugin %s" "Obextool VCal-Plugin %s"\
  "New VCal Entry: %s" "Nowy wpis VCal: %s"\
  "Edit VCal Entry: %s" "Edytuj wpis VCal"\
  "VCal overview" "Podgląd VCal"\
  "On Siemens mobiles, all uploaded" "W telefonach Siemens wszystkie wysyłane"\
  " entries are deactivated by default." " wpisy są dezaktywowane domyślnie."\
  " Don't forget to activate any modified" " Nie zapomnij aktywować zmienionych"\
  " entry in the options of your phone directly." " wpisów w opcjach Twojego telefonu"\
  "No record selected for copying!" "Nie zaznaczono rekordu do skopiowania!"\
  "No record selected for editing!" "Nie zaznaczono rekordu do edycji!"\
  "No record selected for deleting!" "Nie zaznaczono rekordu do usunięcia!"\
  "Do you really want to delete the appointment entry %s?" "Czy na pewno chcesz usunąć wpis o spotkaniu %s?"\
  "Deleting appointment entry" "Usuwanie wpisu o spotkaniu"\
  "Schedule entry %s could not be deleted!" "Wpis harmonogramu %s nie mógł być usunięty!"\
]

set vcs_plug_Text_version "1.0"
