#
#  PL language file for Siemens SMS-Plugin
#  Provided by Kamil Nowak
#

array set adr_plug_Text [list\
  "Address detail view" "Widok szczegółowy adresów"\
  "Address book entry" "Wpis do książki adresowej"\
  "Address list: %s" "Lista adresów: %s"\
  "Address list" "Lista adresów"\
  "%d index entries found," "%d wpisów znaleziono"\
  "No valid pointer file," "Nieprawidłowy wskaźnik pliku"\
	" analyzing address book structure..." " analizowanie struktury książki adresowej..."\
  "Number of entries from both files do not match!" "Pewne wpisy z obu plików nie pasują"\
  "\n%d entries found in '%s' and %d entries in '%s'" "\n%d wpisów znaleziono w '%s' i %d wpisów w '%s'"\
  "\nDeleted entries may be displayed." "\nSkasowane wpisy mogą być wyświetlone."\
  "Processing address book contents..." "Przeszukiwanie zawartości książki adresowej..."\
  "Processing address book contents...OK." "Przeszukiwanie zawartości książki adresowej...Zakończone pomyślnie."\
  "Invalid adress file or unsupported file format." "Nieprawidłowy plik adresów lub niewłaściwy format pliku."\
  "\nUsually on Siemens phones only the file" "\nZwykle tylko na telefonach Siemens"\
  " called '5fxx.adr' contains the address book" "plik '5fxx.adr' zawiera dane książki adresowej."\
  " data. '7fxx.adr' contains the pointer list" "'7fxx.adr' zawiera listę wskaźników"\
  " which mark deleted entities and '9fxx.adr'" "który zaznacza usunięte wpisy i '9fxx.adr'"\
  " contains sort index entries." " zawiera posortowane według indeksu wpisy."\
  "\nTry selecting the file '5fxx.adr' for displaying" "\nSpróbuj wybrać plik '5fxx.adr' aby wyświetlić"\
  " the address book data." " książkę adresową z danymi."\
  "%s window already open!\n" "%s okno jest już otwarte!\n"\
  "Please close it before opening a new one." "Proszę zamknij je  jeśli chcesz  otworzyć nowe."\
  "Unable to download file '%s'!" "Nie można ściągnąć pliku '%s'!"\
  "No record selected for displaying!" "Nie zaznaczono rekordu do wyświetlenia!"\
  "&Backward" "&Cofnij"\
  "&Forward" "&Dalej"\
  "&Show" "&Pokaż"\
  "&Close" "%Zamknij"\
  "E&xport..." "E&ksportuj..."\
  "Go to previous address record" "Idź do porzedniego rekordu adresu"\
  "Go to next address record" "Idź do kolejnego rekordu adresu"\
  "Close detail view" "Zamknij widok szczegółowy"\
  "Show address detail view" "Pokaż szczegółowy widok adresów"\
  "Export data to text file" "Eksportuj dane do pliku tekstowego"\
  "Close address list" "Zamknij listę adresów"\
  "Close detail window before closing list" "Zanim zamkniesz listę zamknij okno z widokiem szczegółowym"\
  "ASCII Text Files <TAB>" "Pliki teksowe ASCII <TAB>"\
  "ASCII Files <Semicolon>" "Pliki ASCII <Średnik>"\
  "CSV Files <Comma separated>" "Pliki CSV <Oddzielone przecinkiem>"\
  "All files" "Wszystkie pliki"\
  "Export file" "Eksportuj plik"\
  "%d records written to file '%s'" "%d rekordów zapisanych do pliku '%s'"\
  "Nr." "Nr."\
  "Firstname" "Imię"\
  "Lastname" "Nazwisko"\
  "Company" "Firma"\
  "Street" "Ulica"\
  "City" "Miasto"\
  "Country" "Kraj"\
  "ZIP" "Kod pocztowy"\
  "Email" "Email"\
  "URL" "URL"\
  "Tel. Home" "Tel. domowy"\
  "Tel. Work" "Tel. praca"\
  "Tel. Mobile" "Tel. komórkowy"\
  "Fax Nr.1" "Fax nr. 1"\
  "Fax Nr.2" "Fax nr. 2" \
  "Birthday" "Data urodzin"\
  "Revision" "Sprawdzanie"\
]

set adr_plug_Text_version "0.2"
