#
#  PL language file for Siemens SMS-Plugin
#  Provided by Kamil Nowak
#

array set siemplug_Text [list\
  "%d minutes" "%d minut"\
  "%d hours" "%d godzin"\
  "%d days" "%d dni"\
  "%d weeks" "%d tygodni"\
  "none" "żaden"\
  "* internal error *" "* błąd wewnętrzny *"\
  "Service center address:" "Adres centrum obsługi:"\
  "Time stamp:" "Znacznik czasu:"\
  "Sender address:" "Adres nadawcy:"\
  "Undocumented header:" "Nieznany nagłówek:"\
  "Incoming SMS file: %s" "Plik SMS-ów przychodzących: %s"\
  "Validy period:" "Okres ważności"\
  "Receiver address:" "Adres odbiorcy:"\
  "Outgoing SMS file: %s" "Plik SMS-ów wychodzących: %s"\
  "No Siemens SMS file signature found!" "Nie znaleziono sygnatury pliku SMS Siemens!"\
  "\nUnable to display SMS text." "\nNie można wyświetlić treści SMS-a."\
  "Unexpected window '%s'!" "Nieoczekiwane okno '%s'!"\
	"Unexpected window class '%s'!" "Nieoczekiwana klasa okna '%s'!"\
  "SMS message saved as '%s'!" "SMS zapisano jako'%s'"\
  "SMS-Text" "Treść SMS-a"\
  "&Close" "&Zakończ"\
  "&Save" "&Zapisz"\
	"Write SMS text into file" "Zapisz treść SMS-a do pliku"\
	"Close SMS window" "Zamknij okno SMS"\
	"SMS text:" "Treść SMS-a:"\
  "SMS data window already open!" "Okno SMS jest już otwarte!"\
  "\nPlease close it before opening a new one." "\nProszę zamknij je zanim otworzysz nowe."\
  "Unable to download file '%s' or file is empty!" "Nie można ściągnąć pliku '%s' lub plik jest pusty!"\
]

set siemplug_Text_version "0.3"
