#!/bin/sh
#
# Shell wrapper for obexftp 0.10.2+
#
# Christian has changed output (of --list) to stderr, so we must
# "redirect" it back to stdout 
#
# Wrapper commands for:

### Version obexftp 0.10.2 - which I'm still using now...
# obexftp.static -d /dev/modem "$@"
# obexftp -d /dev/modem "$@"

### Version obexftp 0.10.3
# obexftp -t /dev/modem "$@" 2>&1

### Version obexftp 0.10.4 (should also work with bluetooth)
#### obexftp -b <btaddr> -B <channel> "$@" 2>&1
### Bluetooth example (replace bt address and channel
# obexftp -b 00:11:22:33:EE:FF -B 9 "$@"

### Typical serial interface call
# obexftp -t /dev/modem "$@" 2>&1

### In my $HOME stored development version
# $HOME/sw/cprog/obexftp-0.10.4/apps/obexftp -t /dev/modem "$@" 2>/dev/null
# obexftp -t /dev/modem "$@"
### obexftp -b 00:16:20:1A:12:8F "$@"
# $HOME/tmp/obexftp-0.20-cad1/obexftp-0.20 -b 00:16:20:1A:12:8F "$@"
# $HOME/tmp/obexftp-0.20-$HOSTNAME/apps/obexftp -b 00:16:20:1A:12:8F "$@"
# obexftp -b 00:13:FD:05:8C:78 "$@"
# obexftp -b 00:16:20:1A:12:8F -B 7 "$@"
obexftp -b 00:1A:75:59:62:62 -B 7 "$@"

