#
#  PL language file for ObexTool
#  Provided by Kamil Nowak
#

array set obextool_Text [list\
  "Program information" "Informacje o programie"\
  "\nA program to communicate with mobile phones using" "\nProgram do komunikacji z telefonami komórkowymi dzięki"\
   " the OBEX protocol (like Siemens, Ericsson or Nokia)" " protokołowi OBEX (takimi jak Siemens, Ericsson lub Nokia)"\
	 "\nObexTool is licensed using the GNU General Public Licence," "\nObexTool objęty jest licencją GNU General Public Licence,"\
	 "\nsee http://www.gnu.org/copyleft/gpl.html" "\nzobacz na http://www.gnu.org/copyleft/gpl.html"\
  "&File" "&Plik"\
  "A&bout" "&O programie" "Program information" "Informacje o programie"\
  "&Upload" "&Wyślij" "Upload a file" "Wyślij plik"\
  "D&ownload" "&Ściągnij" "Download a file" "Ściągnij plik"\
  "&New" "&Nowy" "Create a new folder" "Stwórz nowy folder"\
  "&Find" "&Znajdź" "Find file or folder" "Znajdź plik lub folder"\
  "&Quit" "&Zakończ" "Exit Obex tool" "Zakończ ObexTool"\
  "&Edit" "&Edytuj"\
  "&Cut" "&Wytnij" "Cut selection" "Wytnij zaznaczone"\
  "&Copy" "&Skopiuj" "Copy selection" "Skopiuj zaznaczone"\
  "&Paste" "&Wklej" "Paste selection" "Wklej zaznaczone"\
  "&Delete" "&Usuń" "Delete selection" "Usuń zaznaczone"\
	"&Rename" "&Zmień nazwę" "Rename selection" "Zmień nazwę zaznaczonego"\
  "&View" "&Widok"\
  "&Folder up" "&Folder wyżej" "Up one folder level" "Wyżej o jeden poziom folderów"\
  "&Reload" "&Odśwież" "Reread folder contents" "Odśwież zawartość folderów"\
  "&Properties" "&Właściwości" "File or folder information" "Informacje o folderze lub pliku"\
  "&Memory" "&Pamięć" "Memory status" "Stan pamięci"\
  "&Options" "&Opcje"\
  "&Toolbar" "&Pasek narzędzi" "Show/hide toolbar" "Pokaż/ukryj pasek narzędzi"\
  "&Large icons" "&Duże ikony" "Display large icons" "Wyświetl duże ikony"\
  "&File list" "&Lista plików" "Display file list" "Wyświetl listę plików"\
  "&Details" "&Szczegóły" "Display file details" "Wyświetl szczegóły pliku"\
	"Version %s" "Wersja %s"\
  "Error message" "Błąd"\
  "Fatal Error:\n%s" "Błąd krytyczny:\n%s"\
  "Warning message" "Uwaga"\
  "\nPlease check your file permissions." "\nSprawdź prawa do pliku."\
  "All Files" "Wszystkie pliki"\
  "Save file" "Zapisz plik"\
  "Read file" "Odczytaj plik"\
  "ObexTool %s - input dialogue" "ObexTool %s - wejście"\
  "ObexTool %s - text viewer" "ObexTool %s - edytor tekstowy"\
  "&Close" "&Zamknij"\
  "Close" "Zamknij"\
	"O&k" "O&k"\
	"&Cancel" "&Anuluj"\
	"Cancel file name input" "Anuluj wstawioną nazwę pliku"\
	"Close text window" "Zamknij okno tekstowe"\
	"Accept file name" "Zaakceptuj nazwę pliku"\
\
  "A folder with the name '%s' already exists!" "Folder '%s' już istnieje!"\
  "\nCannot upload file." "\nNie można wysłać pliku."\
  "File '%s' already exists!" "Plik '%s' nie istnieje!"\
  "\nDo you want to overwrite the existing file?" "\nChcesz zastąpić istniejący plik?"\
  "Name collision" "Błąd nazwy"\
  "File upload cancelled!" "Anulowano wysyłanie pliku!"\
  "File upload failed" "Wysłanie pliku nie powiodło się"\
  "Remove of old version of '%s' failed!" "Usuwanie starszej wersji '%s' nie powiodło się!"\
  "File '%s' uploaded to '%s'" "Plik '%s' wysłano do '%s'"\
  "File '%s' could not be uploaded to '%s'!" "Plik '%s' nie mógł być wysłany do '%s'"\
  "New file name:" "Nowa nazwa pliku:"\
  "New name is identical to old name!" "Nowa nazwa jest taka sama jak stara!"\
  "File '%s' renamed to '%s'." "Zmieniono nazwę pliku '%s' na '%s'"\
  "File or folder '%s' could not be renamed!" "Nie można było zmienić nazwy pliku lub folderu '%s'!"\
  "No file or folder selected to delete!" "Nie zanaczono żadnego pliku lub folderu do usunięcia!"\
  "Do you really want to delete the folder '%s'?" "Czy na pewno chcesz usunąć folder '%s'?"\
  "Do you really want to delete the file '%s'?" "Czy na pewno chcesz usunąć plik '%s'?"\
  "Confirmation" "Potwierdzenie"\
  "Delete action cancelled!" "Anulowano usuwanie!"\
  "File or folder '%s' could not be deleted!" "Plik lub folder '%s' nie mógł być usunięty!"\
  "\nFolders must be empty before deleting." "\nFoldery przed usunięciem muszą być puste."\
  "Folder '%s' deleted." "Folder '%s' usunięty."\
  "File '%s' deleted." "Plik '%s' usunięty."\
  "Total %4.1f\nUsed %4.1f\nFree %4.1f" "Całkowitej %4.1f\nUżywanej %4.1f\nWolnej %4.1f"\
  " Memory status\n" "Stan pamięci"\
  "===============\n" "===============\n"\
  "Memory total: %s bytes\n" "Całkowita pamięć    %s bajtów\n"\
  "Memory used:  %s bytes\n" "Używana pamięć: %s bajtów\n"\
  "Memory free:  %s bytes\n" "Wolna pamięć:    %s bajtów\n"\
  "Device memory status" "Stan urządzenia"\
\
  "It seems, that your device does not" "Wydaje się, że w Twoim urządzeniu" \
  " support the memory status feature." " nie jest możliwe sprawdzenie stanu pamięci." \
  "\nMemory status will be disabled." "\nSprawdzanie stanu pamięci zostanie wyłączone." \
  "Unable to create file '%s' in current folder %s!" "Nie jest możliwe utworzenie pliku '%s' w folderze %s!"\
\
  "Edit file" "Edytuj plik"\
  "View file" "Podgląd pliku"\
  "Display text" "Wyświetl tekst"\
  "Upload file" "Wyślij plik"\
  "Download file" "Ściągnij plik"\
  "Cut file" "Wytnij plik"\
  "Copy file" "Skopiuj plik"\
  "Paste file" "Wstaw plik"\
  "Open" "Otwórz"\
  "Delete" "Skasuj"\
  "Rename" "Zmień nazwę"\
  "New folder" "Nowy folder"\
  "Start plugin" "Uruchom plugin"\
  "%s\nDate: %s\nSize: %s\nUser: %s\nGroup: %s" "%s\nData: %s\nRozmiar: %s\nUżytkownik: %s\nGrupa: %s"\
  "Plugin: %s loaded..." "Plugin: %s załadowany..."\
  "Downloading file '%s'..." "Ściąganie pliku '%s'..."\
  "Uploading file '%s'..." "Wysyłanie pliku '%s'..."\
  "Unable to open file '%s'!" "Nie można otworzyć pliku '%s'!"\
  "Text view of file '%s'..." "Podgląd tekstowy pliku '%s'..."\
  "File: %s" "Plik: %s"\
  "Executing editor '%s'..." "Uruchamianie edytora '%s'..."\
  "Executing viewer '%s'..." "Uruchamianie podglądu '%s'..."\
  "Error in executing: %s %s" "Błąd przy wykonywaniu: %s %s"\
	"Viewer '%s' finished." "Podgląd '%s' zakończony."\
  "\nError message: %s\nError code: %s" "\nBłąd: %s\nBłąd kodu: %s"\
  "Question" "Pytanie"\
  "Do you want to upload the modified file?" "Czy chcesz wysłać zmodyfikowany plik?"\
  "File upload failed" "Nie udało się wysłać"\
  "Remove of old version of '%s' failed!" "Nie udało się usunąć starszej wersji '%s'!"\
	"Uploading file '%s' to '%s'..." "Wysyłanie pliku '%s' do '%s'..." \
  "Moving file '%s' to '%s'..." "Przenoszenie pliku '%s' do '%s'..."\
  "File move failed" "Nie udało się przenieść pliku"\
  "File move from '%s' to '%s' failed!" "Nie udało się przenieść pliku '%s' do '%s' !"\
  "File '%s' stored" "Umieszczono plik '%s' "\
  "No default action specified for file type '%s'!" "Nie przyporządkowano żadnej akcji do pliku '%s'!"\
  "New setup section added to options menu." "Nowa sekcja dodana do menu opcji." \
  "Name" "Nazwa" "File type" "Typ pliku" "Size" "Rozmiar"\
  "Date" "Data" "U-perm" "Prawa U." "G-perm" "Prawa G."\
  "No file selected for downloading" "Nic nie zaznaczono do ściągnięcia"\
  "Only files can be downloaded" "Tylko pliki mogą być ściągnięte"\
  "File member '%s' already exists!" "Użytkownik pliku '%s' już istnieje!"\
  "File download cancelled!" "Anulowano ściąganie pliku!"\
  "Overwriting file '%s'!" "Nadpisanie pliku '%s'!"\
  "Error in downloading file '%s'!" "Błąd podczas ściągania pliku '%s'!"\
  "Download of file '%s' failed!" "Nie udało się ściągnąć pliku '%s' !"\
  "File '%s' saved as '%s'" "Plik '%s' zapisano jako '%s'"\
  "Folder can only be deleted!" "Folder można tylko usunąć!"\
  "Nothing selected for cutting!" "Nic nie zaznaczone do wycięcia!"\
  "Cutting '%s'..." "Wycinanie '%s'..."\
  "Nothing selected for copying!" "Nic nie zaznaczono do kopiowania!"\
  "Copying '%s'..." "Kopiowanie '%s'..."\
  "Nothing selected for pasting!" "Nic nie zaznaczono do wklejania!"\
  "Folder '%s' already exists!\nCannot paste file." "Folder '%s' już istnieje!\nNie można wkleić pliku."\
  "File '%s' already exists in folder '%s'!" "Plik '%s' już istnieje w folderze '%s'!"\
  "You must specify a new file name or cancel the command." "Musisz podać inną nazwę lub anulować."\
  "File '%s' pasted" "Wstawiono plik '%s' "\
  "File paste of '%s' failed!" "Nie udało się wkleić pliku '%s' !"\
  "No file or folder selected!" "Nie zaznaczono żadnego pliku lub folderu!"\
  "%d entries with\n" "%d obiektów w \n"\
  "%d subfolders and\n" "%d podfolderach\n"\
  "%d files use\n" "%d plików używanych\n"\
  "%d bytes of memory" "%d bajtów pamięci"\
  "File properties: %s" "Właściwości pliku"\
  "Properties for File '%s'..." "Właściwości dla pliku '%s'..."\
  "Folder properties: %s" "Właściwości folderu: %s"\
  "Properties for folder '%s'..." "Właściwości folderu '%s'..."\
\
  "Folder tree" "Drzewo folderów"\
  "Folder contents" "Zawartość folderów"\
  "Use *, ? and \[ \] for wildcard matching" "Użyj *, ? i \[ \] w celu znalezienia"\
  "Example: *pic\[a-z\]?\[0-9\].bm\[px\]" "Przykład: *pic\[a-z\]?\[0-9\].bm\[px\]"\
  "File pattern to find:" "Szukanie pliku według wzoru:"\
  "Current directory is: %s" "Obecny folder to: %s"\
  "Stop" "Koniec" "Searching '%s'..." "Szukanie '%s'..."\
  "Checked %d folders and %d files" "Sprawdzono %d folderów i %d plików"\
  "    *** Interrupted ***\n" "   *** Zatrzymano ***\n"\
  "\nSearch time was %d seconds" "\nPrzeszukano w czasie %d sekund"\
  "\nStart folder was: %s" "\nPoczątkowy folder: %s"\
  "\nSearch pattern was: '%s'" "Wzór pliku do szukania: '%s'"\
  "\nNumber of results: %d" "\nLiczba rezultatów: %d"\
  "\nTotally used bytes: %d" "\nCałkowity zajęty obszar w bajtach: %d"\
  "\n==========================\n" "\n==========================\n"\
  "Search results:" "Wynik szukania:"\
  "Searching '%s'..." "Szukanie '%s'..."\
  "%d entries in folder '%s'" "%d wpisów w '%s' folderach"\
  "Folder contents reloaded" "Odświeżono zawartość folderu"\
  "Name of new folder:" "Nazwa nowego folderu:"\
  "Cannot create folder '%s'!" "Nie można utworzyć folderu: '%s'!"\
  "\nA folder with that name already exists." "\nFolder o tej nazwie już istnieje."\
  "File member '%s' already exists!" "Użytkownik pliku '%s' już istnieje!"\
  "Folder '%s' created" "Utworzono folder '%s' "\
  "Could not create folder '%s'!" "Nie można było utworzyć folderu '%s'!"\
  "Already at upper most folder level!" "Nie można przejść do folderu wyżej - najwyższy poziom folderu!"\
  "Unknown type" "Nieznany typ"\
  "%s file" "%s plik" \
  "Folder" "Folder"\
  "Log file" "Plik log."\
  "Text file" "Plik tekstowy"\
  "Animation file" "Animacja"\
  "Address book file" "Plik książki adresowej"\
	"Configuration file" "Plik konfiguracyjny"\
  "Bitmap picture file" "Obraz bitmapa"\
  "Midi ring tone file" "Dźwięk midi"\
  "Java definition file" "Plik definicji javy"\
  "Executable Java file" "Plik wykonawczy java"\
  "Organizer entry file" "Plik organizatora"\
  "Bussiness card file" "Plik wizytówki biznesowej"\
  "SMS incoming file" "Plik SMS-ów przychodzących"\
  "SMS outgoing file" "Plik SMS-ów wychodzących"\
  "Data base file" "Plik bazy danych"\
  "Data file" "Plik danych"\
]

set obextool_Text_version "0.33"

